/*! \file   kmain.c
    \brief  Main kernel loop
    \author Markus L. Noga <noga@inrialpes.fr>
*/
    
/*
 *  The contents of this file are subject to the Mozilla Public License
 *  Version 1.0 (the "License"); you may not use this file except in
 *  compliance with the License. You may obtain a copy of the License at
 *  http://www.mozilla.org/MPL/
 *
 *  Software distributed under the License is distributed on an "AS IS"
 *  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 *  License for the specific language governing rights and limitations
 *  under the License.
 *
 *  The Original Code is legOS code, released March 19, 1999.
 *
 *  The Initial Developer of the Original Code is Markus L. Noga.
 *  Portions created by Markus L. Noga are Copyright (C) 1999
 *  Markus L. Noga. All Rights Reserved.
 *
 *  Contributor(s): Markus L. Noga <noga@inrialpes.fr>
 *                  Kekoa Proudfoot <kekoa@graphics.stanford.edu>
 */

#include "sys/mm.h"
#include "sys/time.h"
#include "sys/tm.h"
#include "sys/irq.h"
#include "sys/direct-sensor.h"
#include "sys/direct-ir.h"
#include "rom/system.h"
#include "rom/sound.h"

#include "direct-button.h"
#include "direct-motor.h"

#include "conio.h"

///////////////////////////////////////////////////////////////////////////////
//
// Variables
//
///////////////////////////////////////////////////////////////////////////////

//! firmware recognition string.
/*! the ROM checks for this string when validating new firmware
*/
unsigned char *firmware_string="Do you byte, when I knock?";


///////////////////////////////////////////////////////////////////////////////
//
// Functions
//
///////////////////////////////////////////////////////////////////////////////

//! the user main()
/*! this is what you supply ;-)
*/
extern int main(void);


//! the beginning of everything
/*! initially called by ROM
*/
void kmain(void) {
	int rc;
	int buttons;
        char asdf;
        
	RESET_IRQ=*((void**)0x0000);		// copy ROM reset IRQ vector
        
	while(1) {
		
		power_init();

#ifndef NO_SYSTEM_TIME
                systime_init();
#endif
                
#ifndef NO_MEMORY_MANAGEMENT	
		mm_init();
#endif

#ifndef NO_TASK_MANAGEMENT
		tm_init();
#endif			

#ifndef NO_DIRECT_IR
		dir_init();
#endif

		lcd_clear();
		lcd_show(man_stand);

#ifndef NO_ASCII
                cputs("legOS");
#else		
# ifndef NO_CONSOLE_IO
		cputc_native_4(0x38);           // l
                cputc_native_3(0x3d);           // e
                cputc_native_2(0x7c);           // g
                cputc_native_1(0x7e);           // O
                cputc_native_0(0x6d);           // S
# else		
		lcd_digit(0);			// initlevel 1. show until
# endif
#endif
		lcd_refresh();			// power key released.
		
//		sound_system(1);		// wakeup sound
		while(PRESSED(button_state(),BUTTON_ONOFF))
			;			// wait for key release
		delay(100);
		
		// kernel main loop.
		do {
			buttons=button_state();
			if(PRESSED(buttons,BUTTON_RUN)) {
#ifndef NO_DIRECT_SENSOR
				ds_init();
#endif
		
#ifndef NO_DIRECT_MOTOR
				dm_init();	// don't waste CPU on motors
#endif						// before the user needs them
			
				
				lcd_show(man_run);
				lcd_digit(1);
				lcd_refresh();
//				sound_system(0);
				
				rc=main();	// run the app.
						// for now, part of the kernel
						// code.
				
				lcd_clear();
				lcd_show(man_stand);
				lcd_int(rc);	// show return code and
				lcd_digit(0);	
				lcd_refresh();
//				sound_system(0);

#ifndef NO_DIRECT_MOTOR	
				dm_shutdown();	// you never know if they've
#endif						// been turned off by the user
				
#ifndef NO_DIRECT_SENSOR
				ds_shutdown();
#endif
						
				while(PRESSED(button_state(),BUTTON_RUN))
					;
				delay(100);
			}
		} while(RELEASED(buttons,BUTTON_ONOFF));

						// erase firmware?
		while(PRESSED(button_state(),BUTTON_ONOFF))
			if(PRESSED(button_state(),BUTTON_PROGRAM))
				rom_reset();

		
//		sound_system(0);		// sleep sound
//		while(sound_playing())		// let it complete.
//			;
//		delay(150);
		
#ifndef NO_DIRECT_IR
		dir_shutdown();
#endif
		
#ifndef NO_SYSTEM_TIME
                systime_shutdown();
#endif
                
		lcd_clear();			// remember, powerdown just
		lcd_refresh();			// puts the CPU to sleep.

		power_off();
	}
}
